-- PiraPlex: The PiraMod AddOn Manager
-- Copyright (C) 2008 Justin Weaver a.k.a Piranther (Visit http://piramod.com for contact)
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

PiraPlex = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "FuBarPlugin-2.0");

PiraPlex.revision = tonumber(string.sub("$Revision: 66762 $", 12, -3));
PiraPlex.date = string.sub("$Date: 2008-03-28 13:14:28 -0800 (Fri, 28 Mar 2008) $", 8, 17);

local isBeta = false;

local _G = getfenv(0);

local RED = "|cFFFF0000";
local GREEN = "|cFF00FF00";
local BLUE = "|cFF0000FF";
local MAGENTA = "|cFFFF00FF";
local YELLOW = "|cFFFFFF00";
local CYAN = "|cFF00FFFF";
local GREY = "|cFF777777";
local WHITE = "|cFFFFFFFF"

local tablet = AceLibrary("Tablet-2.0");

local MYTEXT = {
	helpMsg = "|cff00FF99What's all this 'Mod' and 'Dep' business?|r:\n".."PiraPlex separates AddOns into two categories: |cff00FF00Mods|r and |cff009900Deps|r.  |cff009900Deps|r are AddOns that provide no end-user functionaility, but are required by other AddOns.  Only specific AddOns are recognized as |cff009900Deps|r by PiraPlex.  The rest are considered |cff00FF00Mods|r.  PiraPlex will automatically disable |cff009900Deps|r as needed.  On the other hand, |cff00FF00Mods|r must be manually disabled.\n\n".."|cff00FF99Can I modify my other toon's AddOns?:|r\n".."You can only control AddOns for the toon you are currently on.  Unfortunately, adding this feature is impossible under the current Blizzard API.\n\n".."|cff00FF99I installed a new AddOn, but it's not showing up in PiraPlex!|r\n".."When you install a totally new AddOn, you must completely exit WoW.  I would _advise_ you to do the same when you update a pre-existing mod.\n\n".."|cff00FF99Can I disable these annoying tooltips?|r\n".."Yes, right-click the PiraPlex icon to get the dropdown menu.  Have a look under 'Tooltip Options'\n\n".."|cff00FF99How can I contact you (Piranther)?|r\n".."Visit |cff00FF00http://piramod.com|r and join our forums!\n",
	buttonHelp = {
		["PiraPlexEnableAllButton"] = "Enable All AddOns",
		["PiraPlexDisableAllButton"] = "Disable All AddOns",
		["PiraPlexCleanupDepsButton"] = "This forces a run through the Deps to disable/enable them as required.",
		["PiraPlexRestoreCurrentSelectionsButton"] = "Restore AddOns state to login values (or last UI-Reload).",
		["PiraPlexReloadUIButton"] = "Reload you UI.  This activates any changes you have made.";
		["PiraPlexLockPiraPlexCheckButton"] = "Prevent enable/disable of PiraPlex",
		["PiraPlexLockDepsCheckButton"] = "Prevent manual changes to Deps",
	},
};
	
--
-- FuBar Options
PiraPlex.defaultPosition = "LEFT";
PiraPlex.hasIcon = "Interface\\AddOns\\PiraPlex\\icon";
PiraPlex.defaultMinimapPosition = 181;
PiraPlex.cannotDetachTooltip = true;
PiraPlex.hideWithoutStandby = true;
PiraPlex.independentProfile = true;
PiraPlex.hasNoText = true;

PiraPlex.options = {
   type = "group",
		args = {
		["open"] = {
			name = "Open PiraPlex",
			type = "execute",
			desc = "Open the main PiraPlex window",
			order = 1,
			func = function()
				if ( not PiraPlexFrame:IsShown() ) then
					PiraPlexFrame:Show();
				end
			end,
			usage = "",
		},
		["cleanupDeps"] = {
			name = "Cleanup Deps",
			type = "execute",
			desc = MYTEXT.buttonHelp["PiraPlexCleanupDepsButton"];
			order = 2,
			func = function() PiraPlex:CleanupDeps(); end,
			usage = "",
		},
		["undoChanges"] = {
			name = "Undo Changes",
			type = "execute",
			desc = MYTEXT.buttonHelp["PiraPlexRestoreCurrentSelectionsButton"];
			order = 3,
			func = function() PiraPlex:UndoAllChanges(); end,
			usage = "",
		},
		["options"] = {
			type = "group",
			hidden = false,
			name = "Options",
			desc = "Various settings for PiraPlex",
			order = 4,
			args = {
				["lockDeps"] = {
					name = "Lock Deps",
					type = "toggle",
					desc = "Prevent manual changes to Deps",
					order = 1,
					get = 	function() return PiraPlex.db.profile.lockDeps; end,
					set = 	function(v) PiraPlex:SetlockDeps(v); end,
					usage = "",
				},
				["lockPiraPlex"] = {
					name = "Lock PiraPlex",
					type = "toggle",
					desc = "Prevents you from accidentally disabling PiraPlex",
					order = 2,
					get = 	function() return PiraPlex.db.profile.lockPiraPlex; end,
					set = 	function(v) PiraPlex:SetlockPiraPlex(v); end,
					usage = "",
				},
				["showChatMessages"] = {
					name = "Show Chat Messages",
					type = "toggle",
					desc = "Display an info messages every time you change AddOn settings in PiraPlex",
					order = 3,
					get = 	function() return PiraPlex.db.profile.showChatMessages; end,
					set = 	function(v) PiraPlex.db.profile.showChatMessages = v; end,
					usage = "",
				},
				["showButtonTooltips"] = {
					name = "Show Button Tooltips",
					type = "toggle",
					desc = "Show button help tooltips.",
					order = 4,
					get = function() return PiraPlex.db.profile.showButtonTooltips; end,
					set = function(v) PiraPlex.db.profile.showButtonTooltips = v; end,
					usage = "",
				},
				["showCheckButtonTooltips"] = {
					name = "Show Checkbox Tooltips",
					type = "toggle",
					desc = "Show AddOn-info tooltips when the mouse is hovered over a checkbox in the PiraPlex window",
					order = 5,
					get =	function() return PiraPlex.db.profile.showCheckButtonTooltips; end,
					set = function(v) PiraPlex.db.profile.showCheckButtonTooltips = v; end,
					usage = "",
				},
				["scaleGUI"] = {
					name = "GUI scale",
					type = "range",
					desc = "Resize the GUI",
					order = 6,
					min = 10,
					max = 100,
					step = 1,
					get = function() return PiraPlex.db.profile.scaleGUI; end,
					set = function(v) PiraPlex:ScaleGUI(v); end,
					usage = "",
				},
			},
		},
	},
};

PiraPlex.OnMenuRequest = PiraPlex.options;

PiraPlex:RegisterChatCommand({ "/piraplex" }, PiraPlex.options);

local defaults = {
	lockPiraPlex = true,
	lockDeps = true,
	showCheckButtonTooltips = true,
	showChatMessages = false,
	showButtonTooltips = true,
	scaleGUI = 80;
};
PiraPlex:RegisterDB("PiraPlexDB");
PiraPlex:RegisterDefaults('profile', defaults);

PiraPlex.mydata = {
	addOnCount = 0,
	addOns = {},
	headAddOn = nil,
	headMod = nil,
	headDep = nil,
	currentIcon = 1;
};

function PiraPlex:CSPrint(msg)
	if ( self.db.profile.showChatMessages ) then
		self:Print(msg);
	end
end

function PiraPlex:UpdateHelpFrame()
	PiraPlexHelpScrollText:SetText(MYTEXT.helpMsg);
end

-- Hide or show the PiraPlex frame depending on current visibility
function PiraPlex:TogglePiraPlexFrame()
	if ( PiraPlexFrame:IsShown() ) then
		PiraPlexFrame:Hide();
	else
		PiraPlexFrame:Show();
	end
end

function PiraPlex:FullRefresh()
	self:UpdateAllAddOnInfo();
	self:UpdateAllAddOnContainerFrames();
	self:UpdateHelpFrame();
end

function PiraPlex:ScaleGUI(v)
	self.db.profile.scaleGUI = v;
	PiraPlexFrame:SetScale(v/100);
end

-- return true if the AddOn is required by another _enabled_ AddOn
function PiraPlex:IsRequiredByAnEnabledAddOn(addOn)
	for n,m in pairs(addOn.requiredBy) do
		if ( m.enabled ) then
			return true;
		end
	end
	return false;
end

-- Update the display of all containers in the gui
function PiraPlex:UpdateAllAddOnContainerFrames()
	for n,m in pairs(self.mydata.addOns) do
		if ( (not m.loaded) and m.lod and m.loadable and m.enabled ) then
			m.LoDButton:Show();
		else
			m.LoDButton:Hide();
		end
		if ( m.enabled ) then
			m.CheckButton:SetChecked(true);
		else
			m.CheckButton:SetChecked(false);
		end
		m.CheckButtonContainer:SetBackdropColor(m.backdropColor[1],m.backdropColor[2],m.backdropColor[3]);
		m.CheckButtonContainer:SetBackdropBorderColor(m.backdropBorderColor[1],m.backdropBorderColor[2],m.backdropBorderColor[3]);
		if ( m.statusMsg ) then
			_G["PiraPlexCheckButton"..m.name.."FrameText"]:SetText(m.name.." |cffFFFFFF....... "..m.statusMsg.."|r");
		else
			_G["PiraPlexCheckButton"..m.name.."FrameText"]:SetText(m.name);
		end
	end
end

-- Refresh state-info for all AddOns
function PiraPlex:UpdateAllAddOnInfo()
	local enabled,loadable,reason;
	local changeFlag = false;
	local errFlag = false;
	
	for n,m in pairs(self.mydata.addOns) do
		_,_,_,enabled,loadable,reason,_ = GetAddOnInfo(m.num);
		
		m.enabled = enabled;
		m.loadable = loadable;
		m.reason = reason;
	
		m.loaded = IsAddOnLoaded(m.name);
		m.lod = IsAddOnLoadOnDemand(m.name);

		m.errStatus = false;
	
		if ( not m.loadable ) then
			if ( m.reason == "NOT_DEMAND_LOADED" ) then
				m.backdropColor = { 0,1,1 };
				m.backdropBorderColor = { 0,1,1 };
				m.statusMsg = "Reload to Start";
				changeFlag = true;
			elseif ( m.reason == "DEP_NOT_DEMAND_LOADED" ) then
				m.backdropColor = { 0,1,1 };
				m.backdropBorderColor = { 0,1,1 };
				m.statusMsg = "Reload to Start";
				changeFlag = true;
			elseif ( m.reason == "DEP_DISABLED" ) then
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "Dependancy is Disbled";
				m.errStatus = true;
				errFlag = true;
			elseif ( m.reason == "INTERFACE_VERSION" ) then
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "Interface Version Mismatch";
				m.errStatus = true;
				errFlag = true;
			elseif ( m.reason == "DEP_INTERFACE_VERSION" ) then
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "Dependancy Has Version Mismatch";
				m.errStatus = true;
				errFlag = true;
			elseif ( m.reason == "DEP_MISSING" ) then
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "Dependency Missing";
				m.errStatus = true;
				errFlag = true;
			elseif ( m.reason == "DISABLED" ) then
				-- disabled, so NO status
				m.backdropColor = { 0.5,0.5,0.5 };
				m.backdropBorderColor = { 0.5,0.5,0.5 };
				m.statusMsg = nil;
			else
				-- unknown reason!
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "["..m.reason.."]";
				m.errStatus = true;
				errFlag = true;
			end
		elseif ( m.loaded ) then
			if ( m.enabled ) then
				if ( not self.db.profile.lockPiraPlex or (m.lowerName ~= "piraplex") ) then
					m.backdropColor = { 0,0.8,0 };
					m.backdropBorderColor = { 0,0.8,0 };
					m.statusMsg = "Running";
				else
					m.backdropColor = { 0,0.8,0 };
					m.backdropBorderColor = { 0,0.8,0 };
					m.statusMsg = "Running (Locked)";
				end
			else
				m.backdropColor = { 0,1,1 };
				m.backdropBorderColor = { 0,1,1 };
				m.statusMsg = "Reload to Stop"
				changeFlag = true;
			end
		else
			if ( m.lod ) then
				m.backdropColor = { 0,0.5,0 };
				m.backdropBorderColor = { 0,0.5,0 };
				m.statusMsg = "Load On Demand";
			end
		end
	end
	for n,m in pairs(self.mydata.addOns) do
		for nn,mm in pairs(m.deps) do
			if ( (not m.errStatus) and m.enabled and (not mm.enabled) ) then
				m.backdropColor = { 1,0,0 };
				m.backdropBorderColor = { 1,0,0 };
				m.statusMsg = "Dependancy is Disbled";
				m.errStatus = true;
				errFlag = true;
			end
		end
	end
	
	if ( errFlag ) then
		-- an error exists
		if ( self.mydata.currentIcon ~= 3 ) then
			self:SetIcon("Interface\\AddOns\\PiraPlex\\icon3");
			self.mydata.currentIcon = 3;
		end
	elseif ( changeFlag ) then
		-- changes need to be made
		if ( self.mydata.currentIcon ~= 2 ) then
			self:SetIcon("Interface\\AddOns\\PiraPlex\\icon2");
			self.mydata.currentIcon = 2;
		end
	else
		-- nominal
		if ( self.mydata.currentIcon ~= 1 ) then
			self:SetIcon("Interface\\AddOns\\PiraPlex\\icon");
			self.mydata.currentIcon = 1;
		end
	end
end

-- Force loading of a "Load-On-Demand" AddOn
function PiraPlex:LoadAddOn(addOn)
	local loaded,reason = LoadAddOn(addOn.name);
	if ( not loaded ) then
		self:Print("LoD AddOn could not be loaded [reason: "..reason.."]");
	end
	self:FullRefresh();
end

function PiraPlex:CleanupDeps()
    local dmsg = "";
	local emsg = "";
	local dflag = false;
	local eflag = false;
	local iDidSomething;
	for i=1,self.mydata.addOnCount,1 do
		iDidSomething = false;
		for n,m in pairs(self.mydata.addOns) do
			if ( m.isDep ) then
				if ( m.enabled ) then
					if ( not self:IsRequiredByAnEnabledAddOn(m) ) then
						if ( not dflag ) then
							dflag = true;
						else
							dmsg = dmsg ..", ";
						end
						PiraPlex:DisableAddOn(m);
						iDidSomething = true;
						dmsg = dmsg..m.name;
					end
				else
					if ( self:IsRequiredByAnEnabledAddOn(m) ) then
						if ( not eflag ) then
							eflag = true;
						else
							emsg = emsg ..", ";
						end
						PiraPlex:EnableAddOn(m);
						iDidSomething = true;
						emsg = emsg..m.name;
					end
				end
			end
		end
		if ( not iDidSomething ) then
			break;
		end
	end
	if ( dmsg ~= "" ) then
		self:CSPrint("Deps disabled: "..dmsg);
	else
		self:CSPrint("No deps were disabled.");
	end
	if ( emsg ~= "" ) then
		self:CSPrint("Deps enabled: "..emsg);
	else
		self:CSPrint("No deps were enabled.");
	end
end

-- Revert all AddOn changes to the original login state
function PiraPlex:UndoAllChanges()
	DisableAllAddOns();
	for n,m in pairs(self.mydata.addOns) do
		if ( m.wasEnabledAtLogin ) then
			EnableAddOn(m.name);
		end
	end
	self:FullRefresh();
end

-- Enable all AddOns
function PiraPlex:EnableAllAddOns()
	EnableAllAddOns();
	self:FullRefresh();
	self:Print("All AddOns have been enabled.");
end

-- Disable all AddOns (skip PiraPlex if it's "locked")
function PiraPlex:DisableAllAddOns()
	local extraMsg = "";
	DisableAllAddOns();
	if ( self.db.profile.lockPiraPlex ) then
		EnableAddOn("PiraPlex");
		extraMsg = "(except PiraPlex, which is locked)";
	end
	self:FullRefresh();
	self:Print("All AddOns have been disabled"..extraMsg..".");
end

-- Enable an AddOn and all it's requirements
function PiraPlex:EnableAddOn(addOn)
	local msg = "Enabled AddOn(s): "..addOn.name;
	EnableAddOn(addOn.name);
	for n,m in pairs(addOn.deps) do
		if ( not m.enabled ) then
			msg = msg ..", "..m.name;
			EnableAddOn(m.name);
		end
	end
	self:FullRefresh();
	self:CSPrint(msg);
end

-- Disable an AddOn and any of it's _deps_ that are not being used
function PiraPlex:DisableAddOn(addOn)
	local iDisabledSomething = false;
	local msg = "Disabled AddOn(s): "..addOn.name;
	DisableAddOn(addOn.name);
	self:UpdateAllAddOnInfo();
	for i=1,self.mydata.addOnCount,1 do
		for n,m in pairs(addOn.deps) do
			if ( m.isDep and m.enabled and (not self:IsRequiredByAnEnabledAddOn(m)) ) then
				msg = msg ..", "..m.name;
				DisableAddOn(m.name);
				self:UpdateAllAddOnInfo();
				iDisabledSomething = true;
			end
		end
		if ( not iDisabledSomething ) then
			break;
		end
	end
   self:FullRefresh();
   self:CSPrint(msg);
end

-- EVENT: fubar-icon clicks
function PiraPlex:OnClick()
	if ( IsShiftKeyDown() ) then
		ReloadUI();
	else
		self:TogglePiraPlexFrame()
	end
end

function PiraPlex:SetlockDeps(v)
	self.db.profile.lockDeps = v;
	if ( v ) then
		PiraPlexLockDepsCheckButton:SetChecked(true);
	else
		PiraPlexLockDepsCheckButton:SetChecked(false);
	end
end	

function PiraPlex:LockDepsCheckButtonOnClick()
	self:SetlockDeps(not self.db.profile.lockDeps);
end
	
function PiraPlex:SetlockPiraPlex(v)
	self.db.profile.lockPiraPlex = v;
	if ( v ) then
		PiraPlexLockPiraPlexCheckButton:SetChecked(true);
	else
		PiraPlexLockPiraPlexCheckButton:SetChecked(false);
	end
	self:FullRefresh();
end	
	
function PiraPlex:LockPiraPlexCheckButtonOnClick()
	self:SetlockPiraPlex(not self.db.profile.lockPiraPlex)
end	
	
-- EVENT: checkbox click
function PiraPlex:CheckButtonOnClick(addOn)
	if ( not (self.db.profile.lockDeps and addOn.isDep) ) then
		if ( self.mydata.addOns[addOn.lowerName].enabled ) then
			if ( (not PiraPlex.db.profile.lockPiraPlex) or (addOn.lowerName ~= "piraplex") ) then
				self:DisableAddOn(self.mydata.addOns[addOn.lowerName]);
			end
		else
			self:EnableAddOn(self.mydata.addOns[addOn.lowerName]);
		end
	end
	self:UpdateAllAddOnContainerFrames();
	self:CheckButtonOnEnter(addOn);
end

-- EVENT: display checkbox tooltips on mouse-enter
function PiraPlex:CheckButtonOnEnter(addOn)
	local msg, flag, missingMsg;
	if ( self.db.profile.showCheckButtonTooltips ) then
		PiraPlexTooltip:SetOwner(this,"ANCHOR_BOTTOMLEFT");
		PiraPlexTooltip:AddLine("|cffFF6600TITLE:|r "..(addOn.title or ""),1,1,1,1);
		PiraPlexTooltip:AddLine("|cffFFFF00NOTES:|r "..(addOn.notes or ""),0,1,1,1);
		msg = "";
		flag = false;
		for dn,dm in pairs(addOn.deps) do
			if ( not flag ) then
				flag = true;
			else
				msg = msg ..", ";
			end
			if ( (not dm.enabled) or ((not dm.loadable) and (dm.reason ~= "NOT_DEMAND_LOADED") and (dm.reason ~= "DEP_NOT_DEMAND_LOADED")) ) then
				msg = msg .."|cffFF9900"..dm.name.."|r";
			else
				msg = msg .."|cff00FF00"..dm.name.."|r";
			end
		end
		missingMsg = table.concat(addOn.missingDeps,"|r, |cffFF0000")
		if ( missingMsg ~= "" ) then
			if ( msg ~= "" ) then
				comma = ", ";
			else
				comma = false;
			end
			msg = msg ..(comma or "").."|cffFF0000"..missingMsg;
		end
		PiraPlexTooltip:AddLine("DEPS: "..msg,1,1,0,1);
		PiraPlexTooltip:Show();
	end
end

-- EVENT: display button tooltips on mouse-enter
function PiraPlex:ButtonOnEnter()
	if ( self.db.profile.showButtonTooltips ) then
		PiraPlexTooltip:SetOwner(this,"ANCHOR_RIGHT");
		PiraPlexTooltip:AddLine(MYTEXT.buttonHelp[this:GetName()],0,1,1,1);
		PiraPlexTooltip:Show();
	end
end

-- EVENT: hide button tooltips on mouse-leave
function PiraPlex:CheckButtonOnLeave()
	PiraPlexTooltip:Hide();
end

-- EVENT: show fubar-icon tooltip
function PiraPlex:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		'columns', 1,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_justify', "CENTER"
	);

	cat:AddLine('text', YELLOW.."Version|r "..WHITE..self.version.."|r");
	if ( self.mydata.currentIcon == 3 ) then
		cat:AddLine('text', " ");
		cat:AddLine('text', "|cffFF0000An error condition exists in your AddOn settings.|r"); 
	elseif ( self.mydata.currentIcon == 2 ) then
		cat:AddLine('text', " ");
		cat:AddLine('text', "|cff00FFFFA UI-reload is required to finalize your AddOn settings.|r");
	end

	tablet:SetHint("Right-Click for options. Left-Click to open PiraPlex. Shift-Left-Click to reload your UI.");
end



---
-- INIT
---

-- Create a Checkbox in the GUI for the specified AddOn
function PiraPlex:CreateCheckButton(addOn)

	local parentFrame;
	if ( not addOn.isDep ) then
		parentFrame = PiraPlexFrameScrollText;
	else
		parentFrame = PiraPlexDepsScrollText;
	end

	local point, relativeTo, xOffset, yOffset, width, height, lastContainerFrame;
	if ( not addOn.lastLink ) then
		lastContainerFrame = parentFrame;
		point = "TOPLEFT";
		relativeTo = "TOPLEFT";
		xOffset = 5;
		yOffset = 0;
		width = lastContainerFrame:GetWidth();
		height = 30;
	else
		lastContainerFrame = addOn.lastLink.CheckButtonContainer;
		point = "TOPLEFT";
		relativeTo = "BOTTOMLEFT";
		xOffset = 0;
		yOffset = 0;
		width = lastContainerFrame:GetWidth();
		height = lastContainerFrame:GetHeight();
	end

	local CheckButtonContainerFrameName = "PiraPlex"..addOn.name.."CheckButtonContainerFrame";
	addOn.CheckButtonContainer = CreateFrame("Frame",CheckButtonContainerFrameName,parentFrame);

	addOn.CheckButtonContainer:SetPoint(point, lastContainerFrame, relativeTo, xOffset, yOffset);
	addOn.CheckButtonContainer:SetWidth(width);
	addOn.CheckButtonContainer:SetHeight(height);

	addOn.CheckButtonContainer:SetBackdrop(
		{
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true,
			tileSize = 16,
			edgeSize = 16,
			insets = {
				left = 4,
				right = 4,
				top = 4,
				bottom = 4,
			},
		}
	);

	local CheckButtonFrameName = "PiraPlexCheckButton"..addOn.name.."Frame"
	addOn.CheckButton = CreateFrame("CheckButton",CheckButtonFrameName,addOn.CheckButtonContainer,"OptionsCheckButtonTemplate");
	addOn.CheckButton:SetPoint("TOPLEFT", addOn.CheckButtonContainer,"TOPLEFT",0,0);
	addOn.CheckButton:SetScript("OnClick", function() self:CheckButtonOnClick(addOn); end);
	addOn.CheckButton:SetScript("OnEnter", function() self:CheckButtonOnEnter(addOn); end);
	addOn.CheckButton:SetScript("OnLeave", function() self:CheckButtonOnLeave(addOn); end);

	local LoDButtonFrameName = "PiraPlexLoD"..addOn.name.."Button";
	addOn.LoDButton = CreateFrame("Button",LoDButtonFrameName,_G[CheckButtonFrameName],"UIPanelButtonTemplate");
	addOn.LoDButton:SetPoint("RIGHT",addOn.CheckButtonContainer,"RIGHT",-3,0);
	addOn.LoDButton:SetWidth(80);
	addOn.LoDButton:SetHeight(24);
	addOn.LoDButton:SetText("Load Now");
	addOn.LoDButton:SetScript("OnClick", function() PiraPlex:LoadAddOn(PiraPlex.mydata.addOns[addOn.lowerName]); end);
end

-- Compile all our AddOn info into a nice big data structure and construct the Mod and Dep tabs of the GUI
function PiraPlex:BuildAddOnTable()
	local name,title,notes,enabled,loadable,reason,security,addOn,lowerName,dName,dm,depList,optionalDepList;
	local orderedAddOnList = {};
	
	self.mydata.addOnCount = GetNumAddOns();
	for i=1,self.mydata.addOnCount,1 do
		name,title,notes,enabled,loadable,reason,security = GetAddOnInfo(i);
		-- AddOns are returned in alphabetcal order by name
		
		lowerName = string.lower(name);
		
		-- I'll explain later. --Doctor Who
		self.mydata.addOns[lowerName] = {
			num = i, -- Give the WoW-API this number as argument to GetModInfo()
			-- toc fields
			name = name,		
			title = title,
			notes = notes,
			-- mod info
			loaded = IsAddOnLoaded(name), -- is loaded
			lod = IsAddOnLoadOnDemand(name), -- capable of being loaded on-demand
			lowerName = lowerName, -- all lower-case name for string matching
			wasEnabledAtLogin = enabled,  -- for 'undo' button on gui
			isDep = false, -- will indicate if an addon is a 'dep'
			deps = {}, -- lists of links to AddOns that are RequiredDeps
			requiredBy = {}, -- lists of links to addOns that have THIS mod set as a RequiredDep
			missingDeps = {}, -- list of names of RequiredDeps that are NOT INSTALLED!!!
			security = security,
			-- links to other addon's data-structures
			nextAddOn = nil,
			lastAddOn = nil,
			nextLink = nil, -- this will be a dep or a mod depending on which path we travel (dep or mod list)
			lastLink = nil,
			-- frames and GUI stuff
			CheckButtonContainer = nil,
			CheckButton = nil,
			LoDButton = nil,
			backdropColor = { 0.3,0.3,0.3 },
			backdropBorderColor = { 1,1,1 },
			statusMsg = nil,
			-- status
			enabled = enabled,
			loadable = loadable,
			reason = reason,
			errStatus = false;
		};
		addOn = self.mydata.addOns[lowerName];
		
		-- check to see if this is a 'dep'
		for j,n in ipairs(PiraPlex_DEPLIST) do
			if ( string.lower(n) == lowerName ) then
				addOn.isDep = true;
			end
		end
	
		table.insert(orderedAddOnList, lowerName);
	end
	
	-- make lists of dep and requiredBy for each addon
	for n,m in pairs(self.mydata.addOns) do
		-- convert depNames into links
		depList = { GetAddOnDependencies(m.name) };
		for i,dn in ipairs(depList) do
			dName = string.lower(dn);
			dm = self.mydata.addOns[dName];
			if ( dm ) then
				if ( not m.deps[dName] ) then
					m.deps[dName] = dm;
					if ( not dm.requiredBy[n] ) then
						dm.requiredBy[n] = m;
					end
				end
			else
				if ( not string.match(dName,"^blizzard_") ) then
					table.insert(m.missingDeps, dn);
					self:Print("|cffFF0000WARNING:|r AddOn |cff00FFFF"..dn.."|r is not installed!  It is required by |cff00FFFF"..m.name..".|r");
				else
					--self:Print("required blizz frame:"..addOn.name);
				end
			end
		end
	end	
	
	-- Fill out each AddOn's 'deps' table so that it contains deps of deps of deps etc.
    local flag;
	for i=1,self.mydata.addOnCount,1 do
		flag = false;
		for n,m in pairs(self.mydata.addOns) do
			for n1,m1 in pairs(m.deps) do
				for n2,m2 in pairs(m1.deps) do
					if ( (m2 ~= m) and (not m.deps[n2]) ) then
						m.deps[n2] = m2;
						flag = true;
					end
				end
			end
		end
		if ( not flag ) then
			break;
		end
	end
	
	-- Setup the AddOn's linked-lists for AddOns, Deps, and Mods.  Lists are in alphabetacial order by addon name.
	table.sort(orderedAddOnList, function(a,b) return string.lower(a) < string.lower(b); end);
	local lastAddOn = nil;
	local lastDep = nil;
	local lastLink = nil;
	for i,n in ipairs(orderedAddOnList) do
		addOn = self.mydata.addOns[n];

		if ( not self.mydata.headAddOn ) then
			self.mydata.headAddOn = addOn;
		end

		addOn.lastAddon = lastAddOn;
		if lastAddOn then
			lastAddOn.nextAddOn = addOn;
		end
		lastAddOn = addOn;

		if ( not addOn.isDep ) then
			if ( not self.mydata.headMod ) then
				self.mydata.headMod = addOn;
			end
			addOn.lastLink = lastLink;
			if ( lastLink ) then
				lastLink.nextLink = addOn;
			end
			lastLink = addOn;
		else
			if ( not self.mydata.headDep ) then
				self.mydata.headDep = addOn;
			end
			addOn.lastLink = lastDep;
			if ( lastDep ) then
				lastDep.nextLink = addOn;
			end
			lastDep = addOn;
		end
	end
end			

function PiraPlex:OnEnable(first)
	self:BuildAddOnTable();

	-- populate the Mod frame
	local addOn = self.mydata.headMod;
	while ( addOn ) do
		self:CreateCheckButton(addOn);
		addOn = addOn.nextLink;
	end
	-- populate the Dep frame
	addOn = self.mydata.headDep;
	while ( addOn ) do
		self:CreateCheckButton(addOn);
		addOn = addOn.nextLink;
	end
	
	if ( self.db.profile.lockDeps ) then
		PiraPlexLockDepsCheckButton:SetChecked(true);
	else
		PiraPlexLockDepsCheckButton:SetChecked(false);
	end

	if ( self.db.profile.lockPiraPlex ) then
		PiraPlexLockPiraPlexCheckButton:SetChecked(true);
	else
		PiraPlexLockPiraPlexCheckButton:SetChecked(false);
	end

	PiraPlexFrameTopBarVersionText:SetText("|cffFFFF00PiraPlex|r");
	PiraPlexFrameTopBarTitleText:SetText("|cffFFFFFFAddOns for |r|cff00AA00"..(GetUnitName("player") or "?").."|r |cffFFFFFFof|r |cff00ff00"..(GetRealmName() or "?").."|r");

	PiraPlexLockDepsCheckButtonText:SetText("Lock Deps");
	PiraPlexLockPiraPlexCheckButtonText:SetText("Lock PiraPlex");
	
	PiraPlexFrame:SetScale(self.db.profile.scaleGUI/100);
	PiraPlexTooltip:SetScale((self.db.profile.scaleGUI/100)*0.8);
	
	self:FullRefresh();
	
	-- attempt to fix any dep problem when we log in
	PiraPlex:CleanupDeps()
end

function PiraPlex:OnInitialize()
	if ( self.revision and isBeta ) then
		self.version = self.version.."-r"..self.revision;
	end
end
